package fr.iut.mapping

import android.location.Location
import android.location.LocationListener
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.core.content.ContextCompat
import androidx.core.content.res.ResourcesCompat
import androidx.fragment.app.Fragment
import com.google.gson.JsonParser
import com.mapbox.mapboxsdk.Mapbox
import com.mapbox.mapboxsdk.camera.CameraUpdateFactory
import com.mapbox.mapboxsdk.geometry.LatLng
import com.mapbox.mapboxsdk.maps.MapView
import com.mapbox.mapboxsdk.plugins.annotation.Symbol
import com.mapbox.mapboxsdk.plugins.annotation.SymbolManager
import com.mapbox.mapboxsdk.plugins.annotation.SymbolOptions
import com.mapbox.mapboxsdk.utils.BitmapUtils
import fr.iut.mapping.API.RestaurantData
import fr.iut.mapping.API.getRestaurants
import kotlinx.android.synthetic.main.fragment_map_page.*
import kotlinx.coroutines.runBlocking


//https://maplibre.org/maplibre-gl-native/android/api/index.html
class FirstFragment: Fragment(R.layout.fragment_map_page) {
    companion object {
        private const val MARKER_SELECTED_ICON = "JAWG_ICON"
        private const val MARKER_ICON = "MARKER_ICON"
    }

    private var mapView: MapView? = null
    private var symbolManager: SymbolManager? = null
    private var lastSymbol: Symbol? = null

    private lateinit var locationListener: fr.iut.mapping.LocationListener
    private lateinit var listRestaurant: List<RestaurantData>

    private fun makeStyleUrl(): String {
        return "${getString(R.string.mapbox_style_url)}";
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreate(savedInstanceState)

        locationListener = LocationListener(requireContext())
        locationListener.start()


        Mapbox.getInstance(requireContext(), R.string.mapbox_access_token.toString())
        val rootView = inflater.inflate(R.layout.fragment_map_page, container, false)

        val button: View = rootView.findViewById(R.id.findRestaurant)
        button.setOnClickListener { view ->
            findAndPutRestaurant()
        }


        mapView = rootView.findViewById(R.id.mapView)
        mapView?.onCreate(savedInstanceState)

        mapView?.getMapAsync { map ->
            map.setStyle(makeStyleUrl()) { style ->
                map.uiSettings.setAttributionMargins(15, 0, 0, 15)
            }
        }

        return rootView
    }

    private fun findAndPutRestaurant(){
        Log.e("Debug","${locationListener.latitude},${locationListener.longitude}")
        listRestaurant = getRestaurants(locationListener.latitude,locationListener.longitude,1000)


        if(listRestaurant.isEmpty()){
            Toast.makeText(requireContext(), "Pas de restaurants trouver", Toast.LENGTH_SHORT).show()
        }

        mapView?.getMapAsync { map ->
            map.animateCamera(CameraUpdateFactory.newLatLngZoom(LatLng(locationListener.latitude, locationListener.longitude), 11.0))

            map.setStyle(makeStyleUrl()) { style ->
                map.uiSettings.setAttributionMargins(15, 0, 0, 15)
                // Add an icons ("classic" and "selected") to the map style.
                val selectedMarkerIconDrawable = ResourcesCompat.getDrawable(this.resources, R.drawable.ic_menu_likes, null)
                style.addImage(MARKER_ICON, BitmapUtils.getBitmapFromDrawable(selectedMarkerIconDrawable)!!)
                val markerIconDrawable = ResourcesCompat.getDrawable(this.resources, R.drawable.ic_menu_likes, null)
                style.addImage(MARKER_SELECTED_ICON, BitmapUtils.getBitmapFromDrawable(markerIconDrawable)!!)

                // Initialize SymbolManager.
                this.symbolManager = SymbolManager(mapView!!, map, style)
                this.symbolManager?.iconAllowOverlap = true
                this.symbolManager?.iconIgnorePlacement = true

                listRestaurant.forEach { restaurant ->
                    val description = "${restaurant.adress}\n${restaurant.phone ?: "Pas de numéro disponible"}\n${restaurant.lat},${restaurant.lon}"
                    insertIconOnMap(
                        LatLng(restaurant.lat, restaurant.lon),
                        restaurant.name,
                        description
                    )
                }


                // Add a listener to trigger markers clicks.
                this.symbolManager?.addClickListener {
                    // Put all marker information into the layout.
                    titleView.text = it.data?.asJsonObject?.get("title")?.asString
                    descriptionView.text = it.data?.asJsonObject?.get("description")?.asString
                    it?.data?.asJsonObject?.get("imageId")?.asInt?.let { imageId ->
                        descriptionLayout.background = ContextCompat.getDrawable(requireContext(), imageId)
                    }
                    if (descriptionLayout != null && descriptionLayout.background != null) {
                        descriptionLayout.background.alpha = 30
                    }

                    // Set the new marker as selected and toggle layout.
                    setSelectedIcon(it)
                    toggleLayout()
                    true
                }

            }
        }

    }

    private fun toggleLayout() {
        mapLayout?.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 0, 2f)
        descriptionLayout?.layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 0, 1f)
    }

    private fun setDefaultIcon(symbol: Symbol) {
        symbol.iconImage = MARKER_ICON
        symbol.iconSize = 1f
        symbolManager?.update(symbol)
    }

    private fun setSelectedIcon(symbol: Symbol) {
        symbol.iconImage = MARKER_SELECTED_ICON
        symbol.iconSize = 1f
        symbolManager?.update(symbol)

        if (this.lastSymbol != null) {
            setDefaultIcon(this.lastSymbol!!)
        }
        this.lastSymbol = symbol
    }

    private fun insertIconOnMap(point: LatLng, title: String, description: String) {
        Log.e("Debug",title)
        // Convert datas of the marker into Json object.
        val jsonData = """
            {
                "title" : "$title",
                "description" : "$description"
            }
        """
        // Add symbol at specified lat/lon.
        val newSymbol = symbolManager!!.create(
            SymbolOptions().withLatLng(LatLng(point.latitude, point.longitude)).withData(JsonParser.parseString(jsonData))
        )
        setDefaultIcon(newSymbol)
    }


    override fun onStart() {
        super.onStart()
        mapView?.onStart()
    }

    override fun onResume() {
        super.onResume()
        mapView?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapView?.onPause()
    }

    override fun onStop() {
        super.onStop()
        mapView?.onStop()
    }

    override fun onLowMemory() {
        super.onLowMemory()
        mapView?.onLowMemory()
    }

    override fun onDestroy() {
        super.onDestroy()
        mapView?.onDestroy()
        locationListener.stop()
    }


}